// 
// このコードは、https://demo.mescius.jp/diodocs/pdf のサンプルの一部として提供されています。
// © MESCIUS inc. All rights reserved.
// 
using System;
using System.IO;
using System.Drawing;
using System.Numerics;
using GrapeCity.Documents.Pdf;
using GrapeCity.Documents.Text;
using GrapeCity.Documents.Common;
using GrapeCity.Documents.Drawing;
using GrapeCity.Documents.Layout;
using GCTEXT = GrapeCity.Documents.Text;
using GCDRAW = GrapeCity.Documents.Drawing;

namespace DsPdfWeb.Demos
{
    // このサンプルでは、GrapeCity.Documents.Drawing.TableRenderer クラスなどを使用して、
    // タイムチャートテーブルを描画する方法を紹介しています。
    public class TimeChartTable
    {
        public int CreatePDF(Stream stream)
        {
            var doc = new GcPdfDocument();
            var p = doc.Pages.Add(new SizeF(doc.PageSize.Height, doc.PageSize.Width));
            var g = p.Graphics;

            DrawTable(g, g.CanvasSize.Width, g.CanvasSize.Height);

            // PDF を保存します。
            doc.Save(stream);
            return doc.Pages.Count;
        }

        class Range
        {
            public Range(double valueLow, double deltaLow, double valueHigh, double deltaHigh)
            {
                ValueLow = valueLow;
                DeltaLow = deltaLow;
                ValueHigh = valueHigh;
                DeltaHigh = deltaHigh;
            }
            public double ValueLow { get; }
            public double DeltaLow { get; } 
            public double ValueHigh { get; } 
            public double DeltaHigh { get; }
        }

        static void DrawTable(GcGraphics g, float pageWidth, float pageHeight)
        {
            var host = new LayoutHost();
            var view = host.CreateView(pageWidth, pageHeight * 0.7f);

            var rt = view.CreateRect();

            // テーブルの長方形を左、右、上、下から均等に配置します。
            rt.AnchorDeflate(null, 10);

            // テーブルのすべての辺が固定されているので、
            // 列には比例の幅を、行には比例の高さを適用できます。
            var ta = new TableRenderer(g,
                rt, FixedTableSides.All,
                rowCount: 10, columnCount: 32,
                gridLineColor: Color.DimGray,
                gridLineWidth: 1);

            var columns = ta.ColumnRects;
            columns[0].SetWidth(120);
            for (int i = 1; i < 32; i++)
            {
                columns[i].SetStarWidth(1f);
            }

            // テーブルのヘッダーは、テーブルの一部です。
            // つまり、最初の行（rows[0]）は、テーブルのヘッダーになります。
            var rows = ta.RowRects;
            rows[0].SetHeight(40);
            rows[1].SetHeight(30);
            for (int i = 2; i < 10; i++)
            {
                rows[i].SetStarHeight(1f);
            }

            var fmt = new TextFormat
            {
                Font = GCTEXT.Font.FromFile(Path.Combine("Resources", "Fonts", "ipag.ttc")),
                FontSize = 16,
                FontSizeInGraphicUnits = true
            };
            var cs = new CellStyle
            {
                TextFormat = fmt,
                TextAlignment = TextAlignment.Center
            };

            var csTitle = new CellStyle(cs)
            {
                Background = true,
                TextFormat = new TextFormat(fmt)
                {
                    FontSize = 16,
                    FontBold = true
                }
            };
            ta.AddCell(csTitle, 0, 0, 1, 32, "小規模事業のマーケティング計画");

            ta.DefaultCellStyle = new CellStyle(cs)
            {
                ParagraphAlignment = ParagraphAlignment.Center
            };
            for (int i = 1; i < 32; i++)
            {
                ta.AddCell(1, i, i.ToString());
            }

            ta.DefaultCellStyle = new CellStyle(cs)
            {
                TextAlignment = TextAlignment.Leading,
                PaddingLeft = 3
            };
            ta.AddCell(2, 0, "事業概要");
            ta.AddCell(3, 0, "マーケティング分析");
            ta.AddCell(4, 0, "マーケティング戦略");
            ta.AddCell(5, 0, "実施計画");
            ta.AddCell(6, 0, "組織体制");
            ta.AddCell(7, 0, "管理体制");
            ta.AddCell(8, 0, "法的側面");
            ta.AddCell(9, 0, "財務計画");

            // 色でチャートを表示するために、背面に色付きの結合セルを追加します。
            AddBand(2, 1, 6, Color.FromArgb(247, 202, 171));
            AddBand(3, 2, 7, Color.FromArgb(179, 198, 231));
            AddBand(4, 5, 6, Color.FromArgb(255, 229, 154));
            AddBand(5, 11, 6, Color.FromArgb(45, 116, 182));
            AddBand(6, 17, 6, Color.FromArgb(255, 155, 155));
            AddBand(7, 18, 6, Color.FromArgb(197, 224, 179));
            AddBand(8, 24, 4, Color.FromArgb(3, 174, 80));
            AddBand(9, 28, 4, Color.FromArgb(1, 176, 241));

            void AddBand(int rowIndex, int columnIndex, int columnCount, Color color)
            {
                ta.AddCell(new CellStyle
                {
                    Background = true,
                    FillColor = color
                }, rowIndex, columnIndex, 1, columnCount);
            }

            // テーブルのヘッダーのグリッド線を描画しないよう、
            // 最初の行を除いて、不足しているセルを追加します。
            ta.AddMissingCells(1, 0, 9, 32);

            ta.Render();
        }
    }
}
